library("doMC")
library("doSNOW")
library("bigmemory")

compareParallelBackends <- function(n, cores) {

  # Create the matrix.
  a <- matrix(runif(n*n), nrow=n, ncol=n)
  print(object.size(a), unit="Mb")

  # Create a big.matrix copy.
  constTime <- system.time(y <- as.big.matrix(a, backingfile="test.back",
    descriptorfile="test.desc"))

  # Select half of the columns.
  sel <- sample(nrow(a), floor(nrow(a)/2))

  # Perform 10 bootstrap replications.
  bsReps <- 10

  # Serial
  timeser <- system.time(ress <- 
    lapply(sel, 
    function(x) {
      bsMeds <- rep(NA, bsReps)
      for (j in 1:bsReps) {
        bsMeds[j] <- as.numeric(quantile(sample(a[,x], nrow(a), replace=TRUE), 
          probs=0.5))
      }
      c(mean(bsMeds), sd(bsMeds))
    }))
  gc()
  print("finished serial")

  # foreach (with multicore)
  registerDoMC(cores)
  timey <- system.time( resy <- 
    foreach(x=sel) %dopar% {
      bsMeds <- rep(NA, bsReps)
      for (j in 1:bsReps) {
        bsMeds[j] <- as.numeric(quantile(sample(y[,x], nrow(y), replace=TRUE),
          probs=0.5))
      }
      c(mean(bsMeds), sd(bsMeds))
    })
  gc()
  print("finished foreach multicore")

  # mclapply (multicore)
  timea <- system.time( resa <- 
    mclapply(X=sel, mc.cores=cores, 
    FUN=function(x) {
      bsMeds <- rep(NA, bsReps)
      for (j in 1:bsReps) {
        bsMeds[j] <- as.numeric(quantile(sample(a[,x], nrow(a), replace=TRUE), 
          probs=0.5))
      }
      c(mean(bsMeds), sd(bsMeds))
    }))
  gc()
  print("finished mclapply")
 
  # parLapply (with snow) 
  cl <- makeSOCKcluster( rep("localhost", cores) )
  timeb <- system.time( resb <- 
    parLapply(cl, sel, 
    function(x, a, bsReps) {
      bsMeds <- rep(NA, bsReps)
      for (j in 1:bsReps) {
        bsMeds[j] <- as.numeric(quantile(sample(a[,x], nrow(a), replace=TRUE),
          probs=0.5))
      }
      c(mean(bsMeds), sd(bsMeds))
    }, a=a, bsReps=bsReps))
  gc()
  print("finished parLapply")

  # foreach (with snow)
  registerDoSNOW(cl)
  ydesc <- describe(y)
  timez <- system.time( resz <- 
    foreach(x=sel) %dopar% {
      bsMeds <- rep(NA, bsReps)
      for (j in 1:bsReps) {
        require(bigmemory)
        y <- attach.big.matrix(ydesc)
        bsMeds[j] <- as.numeric(quantile(sample(y[,x], nrow(y), replace=TRUE),
          probs=0.5))
      }
      c(mean(bsMeds), sd(bsMeds))
    })
  gc()
  print("finished foreach snow ")
  stopCluster(cl)

  f <- paste(object.size(a))
  c(timeser[3], timea[3], timeb[3], timey[3], timez[3], as.numeric(f))
}
test <- floor(sqrt(c(25000000, 50000000, 75000000, 100000000, 125000000)))

res <- NULL
for (i in test) {
  res <- rbind(res, compareParallelBackends(i, cores=4))
  gc()
}
res <- as.data.frame(res)
names(res) <- c("serial", "mclapply", "parLapply", "foreachm", "foreachs", 
  "object.size")
write.csv(res, "fourcoretiming.csv", row.names=FALSE)

res <- NULL
for (numCores in c(2:6)) {
  res <- rbind(res, compareParallelBackends(n=test[2], numCores))
  gc()
}
res <- as.data.frame(res)
names(res) <- c("serial", "mclapply", "parLapply", "foreachm", "foreachs",
  "object.size")
write.csv(res, "cpuscaling.csv", row.names=FALSE)

